#!/usr/bin/perl -w
use strict;
package ScribbleArea;
use Qt;
use Qt::debug;
use Qt::GlobalSpace qw( bitBlt );
use Qt::isa qw(Qt::Widget);
use Qt::slots
	setColor => ['QColor'],
	slotLoad => ['const QString&'],
	slotSave => ['const QString&'],
	slotClearArea => [];
use Qt::attributes qw(
	_buffer
	_last
	_currentcolor
	_popupmenu
);

 #
 # A class that lets the user draw with the mouse. The
 # window knows how to redraw itself.
 #

		  #
		  # The constructor. Initializes the member variables.
		  #
		sub NEW {
   			shift->SUPER::NEW(@_);
			# initialize member variables
			_buffer = Qt::Pixmap();
			_last = Qt::Point();
			_currentcolor = black();
			
			# don't blank the window before repainting
			setBackgroundMode( &NoBackground );
			
			# create a pop-up menu
			_popupmenu = Qt::PopupMenu();
			_popupmenu->insertItem( "&Clear", this, SLOT( "slotClearArea()" ) );
		}
		
		  #
		  # This slot sets the curren color for the scribble area. It will be
		  # connected with the colorChanged( Qt::Color ) signal from the
		  # ScribbleWindow.
		  #
		sub setColor( $ )
		{
			my ($new_color) = @_;
			_currentcolor = $new_color;
		}
		
		 #
		 # This slot clears the drawing area by filling the off-screen buffer with
		 # white and copying it over to the window.
		 #
		sub slotClearArea
		{
			# fill the off screen buffer with plain white
			_buffer->fill( white() );
			
			# and copy it over to the window
			bitBlt( this, 0, 0, _buffer );
		}
		
		
		  #
		  # This method does the actual loading. It relies on Qt::Pixmap (and the
		  # underlying I/O machinery) to determine the filetype.
		  #
		sub slotLoad( $ )
		{
			my ($filename) = @_;
			if ( !_buffer->load( $filename ) ) {
				Qt::MessageBox::warning( undef, "Load error", "Could not load file" );
			}
				
			repaint();  # refresh the window
		}
		
		
		  #
		 # This method does the actual saving. We hard-code the file type as
		 # BMP. Unix users might want to replace this with something like XPM.
		 #
		sub slotSave( $ )
		{
			my ($filename) = @_;
			if (!_buffer->save( $filename, "BMP" )) {
				Qt::MessageBox::warning( undef, "Save error", "Could not save file" );
			}
		}
		
		
		  #
		  # This method is called whenever the user presses the
		  # mouse over the window. It just records the position of the mouse
		  # at the time of the click.
		  #
		sub mousePressEvent( $ )
		{
			my ($event) = @_;
			if ( $event->button() == &RightButton ) {
				_popupmenu->exec( Qt::Cursor::pos() );
			} else {
				_last = $event->pos();	# retrieve the coordinates from the event
			}
		}
		
		
		  #
		  # The method is called whenever the usr moves the mouse
		  # while the mouse button is pressed. If we had called
		  # setMouseTracking(true) before, the method would also be called
		  # when the mouse was moved with any button pressed. We know that
		  # we haven't, and thus don't have to check whether any buttons are
		  # pressed.
		  #
		sub mouseMoveEvent( $ )
		{
			my ($event) = @_;
			# create a Qt::Painter object for drawing onto the window
			my $windowpainter = Qt::Painter();
			# and another Qt::Painter object for drawing int an off-screen pixmap
			my $bufferpainter = Qt::Painter();
			
			# start painting
			$windowpainter->begin( this ); # This painter paints onto the window
			$bufferpainter->begin( _buffer );  # and this one paints in the buffer
		
			# set a standard pen with the currently selected color
			$windowpainter->setPen( _currentcolor );
			$bufferpainter->setPen( _currentcolor );
		
			# draw a line in both the window and the buffer
			$windowpainter->drawLine( _last, $event->pos() );
			$bufferpainter->drawLine( _last, $event->pos() );
		
			# done with painting
			$windowpainter->end();
			$bufferpainter->end();
		
			# remember the current mouse position
			_last = $event->pos();						
		}
		
		  #
		  # This method is called whenever the widget needs
		  # painting, for example when it has been obscured and then revealed again.
		  #
		sub paintEvent( $ )
		{
			my ($event) = @_;
			bitBlt(this, 0, 0, _buffer);
		}
	
		  #
		  # This method get called whenever the widget needs
		  # painting, for example, when it has been obscured and then revealed again.
		  #
		sub resizeEvent( $ )
		{
			my ($event) = @_;
			my $save = Qt::Pixmap( _buffer );
			_buffer->resize( $event->size() );
			_buffer->fill( white() );
			bitBlt( _buffer, 0, 0, $save );
		}

1;

package ScribbleWindow;

use Qt;
use Qt::isa "Qt::Widget";
use Qt::attributes qw[
    _filemenu
    _colormenu
    _helpmenu
    _menubar
    _scrollview
    _scribblearea
];
use Qt::slots
    slotAbout  => [],
    slotAboutQt     => [],
    slotColorMenu    => ['int'],
    slotLoad    => [     ],
    slotSave=> [     ];
use Qt::signals
    colorChanged  => ['QColor'],
    load     => ['const QString&'],
    save => ['const QString&'];
	
use ScribbleArea;
	
use constant COLOR_MENU_ID_BLACK => 0;
use constant COLOR_MENU_ID_RED => 1;
use constant COLOR_MENU_ID_BLUE => 2;
use constant COLOR_MENU_ID_GREEN => 3;
use constant COLOR_MENU_ID_YELLOW => 4;
	
	sub NEW {
   		shift->SUPER::NEW(@_);
		# The next lines build the menu bar. We first create the menus
		# one by one, then add them to the menu bar. #
		_filemenu = Qt::PopupMenu();  # create a file menu
		_filemenu->insertItem( "&Load", this, SLOT( "slotLoad()" ) );
		_filemenu->insertItem( "&Save", this, SLOT( "slotSave()" ) );
		_filemenu->insertSeparator();
		_filemenu->insertItem( "&Quit", Qt::app(), SLOT( "quit()" ) );
		
		_colormenu = Qt::PopupMenu(); # create a color menu
		_colormenu->insertItem( "B&lack", COLOR_MENU_ID_BLACK);
		_colormenu->insertItem( "&Red", COLOR_MENU_ID_RED);
		_colormenu->insertItem( "&Blue", COLOR_MENU_ID_BLUE);
		_colormenu->insertItem( "&Green", COLOR_MENU_ID_GREEN);
		_colormenu->insertItem( "&Yellow", COLOR_MENU_ID_YELLOW);
		Qt::Object::connect( _colormenu, SIGNAL( "activated( int )" ),
						 this, SLOT( "slotColorMenu( int )" ) );
						
		_helpmenu = Qt::PopupMenu();  # create a help menu
		_helpmenu->insertItem( "&About QtScribble", this, SLOT( "slotAbout()" ) );
		_helpmenu->insertItem( "&About Qt", this, SLOT( "slotAboutQt()" ) );
		
		_menubar = Qt::MenuBar( this, "" );  # create a menu bar
		_menubar->insertItem( "&File", _filemenu );
		_menubar->insertItem( "&Color", _colormenu );
		_menubar->insertItem( "&Help", _helpmenu );
		
		 # We create a Qt::ScrollView and a ScribbleArea. The ScribbleArea will
		 # be managed by the scroll view.#
		_scrollview = Qt::ScrollView( this );
		_scrollview->setGeometry( 0, _menubar->height(),
		 							width(), height() - _menubar->height() );
		_scribblearea = ScribbleArea();
		_scribblearea->setGeometry( 0, 0, 1000, 1000 );
		_scrollview->addChild( _scribblearea );
		Qt::Object::connect( this, SIGNAL( "colorChanged(QColor)" ),
						 _scribblearea, SLOT( "setColor(QColor)" ) );
		Qt::Object::connect( this, SIGNAL( "save(const QString&)" ),
						 _scribblearea, SLOT( "slotSave(const QString&)" ) );
		Qt::Object::connect( this, SIGNAL( "load(const QString&)" ),
						 _scribblearea, SLOT( "slotLoad(const QString&)" ) );
	}
	
	sub resizeEvent( $ )
	{
		my ($event) = @_;
		 # When the whole window is resized, we have to rearrange the geometry
		 # in the ScribbleWindow as well. Note that the ScribbleArea does not need
		 # to be changed.
		 _scrollview->setGeometry( 0, _menubar->height(),
		 							width(), height() - _menubar->height() );
	}

	
	
	sub slotAbout
	{
		Qt::MessageBox::information( this, "About QtScribble 5",
										"This is the Scribble 5 application\n" .
										"Copyright 1998 by Mathias Kalle Dalheimer\n");
	}
	
	sub slotAboutQt
	{
		Qt::MessageBox::aboutQt( this, "About Qt" );
	}
	
	sub slotColorMenu( $ )
	{
		my ($item) = @_;
		if ( $item == COLOR_MENU_ID_BLACK ) {
			emit colorChanged(black());
		} elsif ( $item == COLOR_MENU_ID_RED ) {
			emit colorChanged(darkRed());
		} elsif ( $item == COLOR_MENU_ID_BLUE ) {
			emit colorChanged(darkBlue());
		} elsif ( $item == COLOR_MENU_ID_GREEN ) {
			emit colorChanged(darkGreen());
		} elsif ( $item == COLOR_MENU_ID_YELLOW ) {
			emit colorChanged(yellow());
		}
	}
	
	
	  #
	  # This is the slot for the menu item File/Load. It opens a
	  # Qt::FileDialog to ask the user for a filename, then emits a save()
	  # signal with the filename as parameter.
	  #
	sub slotLoad
	{
		 # Open a file dialog for loading. The default directory is the
		 # current directory, the filter *.bmp.
		 #
		my $filename = Qt::FileDialog::getOpenFileName( ".", "*.bmp", this );
		if ( $filename ) {
			emit load($filename);
		}
	}
	
	  #
	  # This is the slot for the menu item File/Load. It opens a
	  # Qt::FileDialog to ask the user for a filename, then emits a save()
	  # signal with the filename as parameter.
	  #
	sub slotSave
	{
		 # Open a file dialog for saving. The default directory is the
		 # current directory, the filter *.bmp.
		 #
		my $filename = Qt::FileDialog::getSaveFileName( ".", "*.bmp", this );
		if ( $filename ) {
			emit save($filename);
		}
	}

1;

package main;

use Qt;
use ScribbleWindow;

my $myapp = Qt::Application(\@ARGV);
my $mywidget = ScribbleWindow();
$mywidget->setGeometry(50, 500, 400, 400);

$myapp->setMainWidget($mywidget);
$mywidget->show();
$myapp->exec();
